Imports System.Drawing.Drawing2D
''' <summary>
''' zeigt einen Tag als Control an
''' </summary>
Public Class DayControl
    Inherits Control

#Region " Private Fields / Events "
    ''' <summary>
    ''' Event bei Auswahl durch die Maus
    ''' </summary>
    Public Event WasSelected(ByVal Ctl As DayControl)

    ' Liste mit Zeitbereichen fr einen einzelnen Tag
    Private WithEvents m_DayTimeAreas As DayTimeAreaList
    ' Auswahl-Zustand des Controls
    Private m_Selected As Boolean = False
    ' MouseOver-Zustand des Controls
    Private m_Hoovered As Boolean = False
    ' OutOfMonth bedeutet, dieses Control nicht zu dem im Monats-Control 
    ' angezeigten Monat gehrt
    Private m_OutOfMonth As Boolean = False
    ' Brush zum Zeichnen des Hintergrundes 
    Private m_BackBrush As LinearGradientBrush



    ' Zeichen-Breite einer Minute
    Private Shared m_MinuteWidth As Single

    ' Hintergrund-Brush fr normale Wochentage
    Private Shared m_NormalDayBrush As LinearGradientBrush
    ' Hintergrund-Brush fr Samstage
    Private Shared m_SaturdayBrush As LinearGradientBrush
    ' Hintergrund-Brush fr Sonntage
    Private Shared m_SundayBrush As LinearGradientBrush
    ' Hintergrund-Brush fr selektierte oder gehooverte Tage
    Private Shared m_SelectionBrush As LinearGradientBrush


    ' Brush zum Zeichnen der Zeitabschnitte
    Private Shared m_LightGreenBrush As SolidBrush
    ' Hintergrund-Brush zum Zeichnen der Zeitlinie
    Private Shared m_LightWhiteBrush As SolidBrush

    ' obere Begrenzung der Zeitlinie
    Private Shared m_TimeLineTop As Single
    ' Hhe der Zeitlinie
    Private Shared m_TimeLineHeight As Single
    ' Rechteck, in dem die Zeitlinie gezeichnet wird
    Private Shared m_TimeLineRect As RectangleF

    ' Fllrechteck fr das Control
    Private Shared m_FillRect As RectangleF
    ' Rahmenrechteck fr das Control
    Private Shared m_BorderRect As Rectangle
#End Region

#Region " Properties "
    ''' <summary>
    ''' Gibt zurck oder legt fest, ob dieses Tages-Control ein Datum 
    ''' auerhalb des im Monats-Control angezeigten Monats enthlt
    ''' </summary>
    Property OutOfMonth() As Boolean
        Get
            Return m_OutOfMonth
        End Get
        Set(ByVal value As Boolean)
            m_OutOfMonth = value
        End Set
    End Property

    ''' <summary>
    ''' Gibt zurck oder legt fest, ob dieses Tages-Control selektiert ist
    ''' </summary>
    Property Selected() As Boolean
        Get
            Return m_Selected
        End Get
        Set(ByVal value As Boolean)
            If m_Selected <> value Then
                m_Selected = value
                ' Nach nderung des Auswahlzustandes Control neu zeichnen
                Invalidate()
            End If
        End Set
    End Property

    ''' <summary>
    ''' gibt die fr die Anzeige verwendete Tages-Zeitbereichsliste zurck
    ''' </summary>
    ''' <remarks>diese Liste enthlt Zeitbereiche fr ein einzelnes Datum</remarks>
    ReadOnly Property DayTimeAreas() As DayTimeAreaList
        Get
            Return m_DayTimeAreas
        End Get
    End Property
#End Region

#Region " Constructor "
    ''' <summary>
    ''' Shared Konstruktor
    ''' </summary>
    ''' <remarks></remarks>
    Shared Sub New()
        ' hier werden gemeinsam genutzte Felder initialisiert
        m_MinuteWidth = 98.0F / 1440.0F
        m_FillRect = New RectangleF(0.0F, 0.0F, 100.0F, 50.0F)
        m_BorderRect = New Rectangle(0, 0, 99, 49)
        m_TimeLineTop = 30.0F
        m_TimeLineHeight = 15.0F
        m_TimeLineRect = New RectangleF(1.0F, m_TimeLineTop, 98.0F, m_TimeLineHeight)

        Dim LightGreenColor As Color = Color.FromArgb(120, Color.LightGreen)
        m_LightGreenBrush = New SolidBrush(LightGreenColor)

        m_NormalDayBrush = New LinearGradientBrush(m_FillRect, Color.White, Color.LightGray, 80.0F)
        m_SaturdayBrush = New LinearGradientBrush(m_FillRect, Color.White, Color.FromArgb(238, 204, 204), 80.0F)
        m_SundayBrush = New LinearGradientBrush(m_FillRect, Color.White, Color.FromArgb(255, 187, 187), 80.0F)

        m_LightWhiteBrush = New SolidBrush(Color.FromArgb(150, Color.White))
        m_SelectionBrush = New LinearGradientBrush(m_FillRect, Color.White, Color.FromArgb(100, Color.DarkBlue), 80.0F)
    End Sub

    ''' <summary>
    ''' Default Konstruktor
    ''' </summary>
    ''' <param name="source">Tages-Zeitbereichsliste</param>
    Public Sub New(ByVal source As DayTimeAreaList)
        ' DoubleBuffer aktivieren
        Me.SetStyle(ControlStyles.DoubleBuffer Or ControlStyles.UserPaint Or ControlStyles.AllPaintingInWmPaint Or ControlStyles.Selectable, True)
        ' Referenz zur Tages-Zeitbereichsliste bernehmen
        ' (diese hier verwendete Listen-Instanz wird spter auch im TimeSelectionControl verwendet)
        m_DayTimeAreas = source

        ' Hintergrund-Brush fr den zum Datum gehrenden Wochentag festlegen
        If m_DayTimeAreas.Datum.DayOfWeek = DayOfWeek.Saturday Then
            m_BackBrush = m_SaturdayBrush
        ElseIf m_DayTimeAreas.Datum.DayOfWeek = DayOfWeek.Sunday Then
            m_BackBrush = m_SundayBrush
        Else
            m_BackBrush = m_NormalDayBrush
        End If
        ' alle Tages-Controls sind 100*50 Px gross
        Me.Size = New Size(100, 50)
    End Sub
#End Region

#Region " Methods "
    ''' <summary>
    ''' berschreiben von OnPaint, um den Inhalt dieses Controls zu zeichnen
    ''' </summary>
    Protected Overrides Sub OnPaint(ByVal e As System.Windows.Forms.PaintEventArgs)
        MyBase.OnPaint(e)

        If m_Selected OrElse m_Hoovered Then
            ' wenn selektiert oder gehoovert, Hintergrund mit blauem Brush fllen
            ' und drber schon mal das TimeLine-Rechteck zeichnen 
            ' (she sonst zu fade aus)
            e.Graphics.FillRectangle(m_SelectionBrush, m_FillRect)
            e.Graphics.FillRectangle(m_LightWhiteBrush, m_TimeLineRect)
        Else
            ' ansonsten Hintergrund mid dem Wochentags-Brush fllen
            e.Graphics.FillRectangle(m_BackBrush, m_FillRect)
        End If

        ' weisse Zeitlinie zeichnen
        e.Graphics.FillRectangle(m_LightWhiteBrush, m_TimeLineRect)

        ' alle Zeitbereiche grn zeichnen
        For Each Bereich As TimeArea In m_DayTimeAreas
            Dim Rect As New RectangleF(CSng(Bereich.Start.Hour * 60 + Bereich.Start.Minute) * m_MinuteWidth + 1.0F, m_TimeLineTop, Bereich.Length * m_MinuteWidth, m_TimeLineHeight)
            e.Graphics.FillRectangle(m_LightGreenBrush, Rect)
        Next

        ' auserhalb des angezeigten Monats liegende Tages-Controls 
        ' erhalten eine leicht graue Beschriftung, die anderen werden
        ' schwarz beschriftet
        If m_OutOfMonth Then
            e.Graphics.DrawString(Format(m_DayTimeAreas.Datum, "ddd") & ", " & m_DayTimeAreas.Datum.ToShortDateString, Me.Font, Brushes.LightGray, 0.0F, 0.0F)
        Else
            e.Graphics.DrawString(Format(m_DayTimeAreas.Datum, "ddd") & ", " & m_DayTimeAreas.Datum.ToShortDateString, Me.Font, Brushes.Black, 0.0F, 0.0F)
        End If

        ' Control-Rahmen zeichnen
        ControlPaint.DrawVisualStyleBorder(e.Graphics, m_BorderRect)
    End Sub

    ''' <summary>
    ''' Handler fr das Changed-Event der Tages-Zeitbereichsliste
    ''' </summary>
    Private Sub m_ZeitAuswahl_Changed() Handles m_DayTimeAreas.Changed
        ' Control neu zeichnen
        Invalidate()
    End Sub

    ''' <summary>
    ''' Handler fr das OnMouseDown-Event dieses Controls
    ''' </summary>
    Protected Overrides Sub OnMouseDown(ByVal e As System.Windows.Forms.MouseEventArgs)
        MyBase.OnMouseDown(e)
        Me.Focus()
        ' Control selektieren
        m_Selected = True
        ' neu zeichnen erzwingen
        Invalidate()
        ' Selected-Event liefern (in der Folge werden andere Controls deselektiert)
        RaiseEvent WasSelected(Me)
    End Sub

    ''' <summary>
    ''' Handler fr das OnMouseEnter-Event dieses Controls
    ''' </summary>
    Protected Overrides Sub OnMouseEnter(ByVal e As System.EventArgs)
        MyBase.OnMouseEnter(e)
        ' Hoover anzeigen
        SetHoovered(True)
    End Sub

    ''' <summary>
    ''' Handler fr das OnMouseLeave-Event dieses Controls
    ''' </summary>
    Protected Overrides Sub OnMouseLeave(ByVal e As System.EventArgs)
        MyBase.OnMouseLeave(e)
        ' Hoover nicht mehr anzeigen
        SetHoovered(False)
    End Sub

    ''' <summary>
    ''' ndert den Hoovered-Zustand dieses Controls
    ''' </summary>
    Private Sub SetHoovered(ByVal NewValue As Boolean)
        If m_Hoovered <> NewValue Then
            m_Hoovered = NewValue
            ' bei nderung des Zustands neu zeichnen
            Invalidate()
        End If
    End Sub
#End Region

End Class
