Imports System.Drawing.Drawing2D
Imports System.ComponentModel
''' <summary>
''' Control zum Auswhlen von Zeibereichen
''' </summary>
Public Class TimeSelectionControl
    Inherits Control

#Region " Private Fields "
    ' verwendete Datenliste, die Zeitbereiche fr einen einzelnen Tag enthlt 
    ' und ein Changed-Event liefern kann
    Private WithEvents m_TimeAreas As New DayTimeAreaList(Date.Today, New List(Of TimeArea))
    ' Hintergrund-Rechteck
    Private m_BackRect As RectangleF
    ' Brush zum Zeichnen des Hintergrunds
    Private m_BackBrush As LinearGradientBrush
    ' grauer Pen
    Private m_LightGrayPen As Pen
    ' grauer Brush
    Private m_LightGrayBrush As Brush
    ' grner Brush fr Zeit-Markierung
    Private m_LightGreenBrush As SolidBrush
    ' oranger Brush fr selektierte Zeit-Markierung
    Private m_SelectionBrush As SolidBrush

    ' Breite einer Viertelstunde
    Private m_ViertelStundeWidth As Single
    ' Breite einer halben Stunde
    Private m_HalbStundeWidth As Single
    ' Breite einer Stunde
    Private m_StundeWidth As Single
    ' Breite einer Minute
    Private m_MinuteWidth As Single

    ' MouseDown-Position
    Private m_MouseDownPosition As Single = -1.0F
    ' MoustPosition bei MouseMove
    Private m_MouseMovePosition As Single
#End Region

#Region " Properties "
    ''' <summary>
    ''' gibt die verwendete Datenliste mit Zeitbereichen zurck oder legt diese fest
    ''' </summary>
    <DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)> _
    Property TimeAreas() As TimeAreaList
        Get
            Return m_TimeAreas
        End Get
        Set(ByVal value As TimeAreaList)
            If m_TimeAreas IsNot value Then
                m_TimeAreas = value
                ' nach nderung Control neu zeichnen
                Invalidate()
            End If
        End Set
    End Property
#End Region

#Region " Constructor "
    Public Sub New()
        ' DoubleBuffer aktivieren
        Me.SetStyle(ControlStyles.DoubleBuffer Or ControlStyles.UserPaint Or ControlStyles.AllPaintingInWmPaint Or ControlStyles.Selectable, True)

        ' Pens und Brushes erstellen
        Dim LightGray As Color = Color.FromArgb(75, Color.Black)
        m_LightGrayPen = New Pen(LightGray)
        m_LightGrayBrush = New SolidBrush(LightGray)

        m_LightGreenBrush = New SolidBrush(Color.FromArgb(150, Color.LightGreen))
        m_SelectionBrush = New SolidBrush(Color.FromArgb(150, Color.OrangeRed))
    End Sub
#End Region

#Region " Methods "
    ''' <summary>
    ''' berschreiben von OnPaint, um den Inhalt des Controls zu zeichnen
    ''' </summary>
    Protected Overrides Sub OnPaint(ByVal e As System.Windows.Forms.PaintEventArgs)
        MyBase.OnPaint(e)

        ' Control-Flache mit Hintergrundbrush fllen
        e.Graphics.FillRectangle(m_BackBrush, m_BackRect)

        ' Makierungsstriche fr Viertel- und ganze Stunden zeichnen
        Dim Left, Bottom As Single
        Bottom = Me.Height
        For I As Single = 1 To 95
            Left = I * m_ViertelStundeWidth
            e.Graphics.DrawLine(Pens.White, Left, 0.0F, Left, Bottom)
        Next
        For I As Single = 1 To 23
            Left = I * m_StundeWidth
            e.Graphics.DrawLine(m_LightGrayPen, Left, 0.0F, Left, Bottom)
        Next


        ' Zeitbereiche zeichnen
        For Each ZB As TimeArea In m_TimeAreas
            Dim StartPos As Single = CSng(ZB.Start.Minute + ZB.Start.Hour * 60) * m_MinuteWidth
            Dim Length As Single = CSng(ZB.Length) * m_MinuteWidth
            Dim Rect As New RectangleF(StartPos, 0.0F, Length, Bottom)
            If m_TimeAreas.SelectedBereich Is ZB Then
                ' selektierter Bereich wird orange gezeichnet
                e.Graphics.FillRectangle(m_SelectionBrush, Rect)
            Else
                ' die anderen Bereiche werden grn gezeichnet
                e.Graphics.FillRectangle(m_LightGreenBrush, Rect)
            End If
        Next

        ' wenn die linke Maustaste gedrckt ist, oranges Markierungs-Rechteck zeichnen
        If m_MouseDownPosition <> -1.0F Then
            If m_MouseMovePosition > m_MouseDownPosition Then
                Dim Rect As New RectangleF(m_MouseDownPosition, 0.0F, m_MouseMovePosition - m_MouseDownPosition, Bottom)
                e.Graphics.FillRectangle(m_SelectionBrush, Rect)
            ElseIf m_MouseMovePosition < m_MouseDownPosition Then
                Dim Rect As New RectangleF(m_MouseMovePosition, 0.0F, m_MouseDownPosition - m_MouseMovePosition, Bottom)
                e.Graphics.FillRectangle(m_SelectionBrush, Rect)
            End If
        End If

        ' Ziffern zur Beschriftung der Stundenmarkierungen zeichnen
        For I As Single = 0 To 23
            Left = I * m_StundeWidth
            e.Graphics.DrawString(I.ToString, Me.Font, Brushes.Black, Left, 0.0F)
        Next

        ' Control-Rahmen zeichnen
        ControlPaint.DrawVisualStyleBorder(e.Graphics, New Rectangle(0, 0, Me.Width - 1, Me.Height - 1))
    End Sub

    ''' <summary>
    ''' berschreiben von OnResize, um grenabhngige Objekte neu zu definieren
    ''' </summary>
    Protected Overrides Sub OnResize(ByVal e As System.EventArgs)
        MyBase.OnResize(e)

        Dim Width As Decimal = Me.Width - 1
        m_ViertelStundeWidth = Width / 96.0F
        m_StundeWidth = Width / 24.0F
        m_HalbStundeWidth = Width / 48.0F
        m_MinuteWidth = Width / 1440.0F

        m_BackRect = New RectangleF(0.0F, 0.0F, Me.Width, Me.Height)
        m_BackBrush = New LinearGradientBrush(m_BackRect, Color.White, Color.LightGray, 90.0F)
    End Sub

    ''' <summary>
    ''' berschreiben von OnMouseDown
    ''' </summary>
    Protected Overrides Sub OnMouseDown(ByVal e As System.Windows.Forms.MouseEventArgs)
        MyBase.OnMouseDown(e)
        ' Control den Focus verleihen (um diesen den anderen Controls, wie z.B. markierter ComboBox zu entziehen)
        Me.Focus()

        ' eventuell neu selektierten Zeitbereich auswhlen
        Dim AcMinute As Integer = Math.Round(CSng(e.X) / m_MinuteWidth)
        Dim AcDate As New Date(m_TimeAreas.Datum.Ticks)
        AcDate = AcDate.AddMinutes(AcMinute)
        For Each Bereich As TimeArea In m_TimeAreas
            If Bereich.Start <= AcDate AndAlso Bereich.Ende >= AcDate Then
                m_TimeAreas.SelectedBereich = Bereich
                Invalidate()
                Exit For
            End If
        Next

        If e.Button <> Windows.Forms.MouseButtons.Left Then Exit Sub
        ' MouseDown-Position fr oranges Markierungs-Rechteck festlegen

        ' gleich die passende Halbstunden-Position auswhlen
        m_MouseDownPosition = Math.Round(CSng(e.X) / m_HalbStundeWidth) * m_HalbStundeWidth
        m_MouseMovePosition = m_MouseDownPosition

        ' Control neu zeichnen
        Invalidate()
    End Sub

    ''' <summary>
    ''' berschreiben von OnMouseMove
    ''' </summary>
    Protected Overrides Sub OnMouseMove(ByVal e As System.Windows.Forms.MouseEventArgs)
        MyBase.OnMouseMove(e)

        If m_MouseDownPosition = -1.0F Then Exit Sub
        ' wenn vorher die linke Maustaste auf dem Control bedient wurde,
        ' neue MouseMovePosition fr oranges Markierungs-Rechteck festlegen
        Dim NewMouseMovePosition As Single = Math.Round(CSng(e.X) / m_HalbStundeWidth) * m_HalbStundeWidth
        If m_MouseMovePosition <> NewMouseMovePosition Then
            m_MouseMovePosition = NewMouseMovePosition
            ' Control neu zeichnen
            Invalidate()
        End If

    End Sub

    ''' <summary>
    ''' berschreiben von OnMouseUp
    ''' </summary>
    Protected Overrides Sub OnMouseUp(ByVal e As System.Windows.Forms.MouseEventArgs)
        MyBase.OnMouseUp(e)
        If m_MouseDownPosition = -1.0F Then Exit Sub

        ' bei MouseUp wird gegebenenfalls ein neuer Zeitbereich zur Liste hinzugefgt

        ' gleich die passende Halbstunden-Position auswhlen
        m_MouseMovePosition = Math.Round(CSng(e.X) / m_HalbStundeWidth) * m_HalbStundeWidth
        m_MouseMovePosition = Math.Max(m_MouseMovePosition, 0.0F)
        m_MouseMovePosition = Math.Min(m_MouseMovePosition, CSng(Me.Width - 1))

        If m_MouseMovePosition <> m_MouseDownPosition Then
            Dim StartMinute As Integer = Math.Round(CSng(Math.Min(m_MouseMovePosition, m_MouseDownPosition)) / m_HalbStundeWidth) * 30
            Dim EndMinute As Integer = Math.Round(CSng(Math.Max(m_MouseMovePosition, m_MouseDownPosition)) / m_HalbStundeWidth) * 30

            If StartMinute <> EndMinute Then
                Dim StartTime As New Date(m_TimeAreas.Datum.Ticks)
                StartTime = StartTime.AddMinutes(StartMinute)
                m_TimeAreas.Add(New TimeArea(StartTime, EndMinute - StartMinute))
            End If
        End If
        ' MouseDownPosition zurck setzen
        m_MouseDownPosition = -1.0F
        ' Control neu zeichnen
        Invalidate()
    End Sub

    ''' <summary>
    ''' berschreiben von OnKeyDown, um auf Button-Ereignisse zu reagieren
    ''' </summary>
    Protected Overrides Sub OnKeyDown(ByVal e As System.Windows.Forms.KeyEventArgs)
        MyBase.OnKeyDown(e)
        ' Bei Keys.Delete den eventuell vorhandenen selektierten Beriech entfernen
        If e.KeyCode = Keys.Delete Then
            If m_TimeAreas.SelectedBereich IsNot Nothing Then
                m_TimeAreas.Remove(m_TimeAreas.SelectedBereich)
            End If
        End If
    End Sub

    ''' <summary>
    ''' berschreiben von OnLostFocus, um den selektierten Bereich zu deselektieren
    ''' </summary>
    Protected Overrides Sub OnLostFocus(ByVal e As System.EventArgs)
        MyBase.OnLostFocus(e)
        m_TimeAreas.SelectedBereich = Nothing
    End Sub

    ''' <summary>
    ''' Handler fr das Changed-Event der Zeitbereichsliste
    ''' </summary>
    Private Sub m_Bereiche_Changed() Handles m_TimeAreas.Changed
        ' Control neu zeichnen
        Invalidate()
    End Sub
#End Region

End Class
