''' <summary>
''' Auflistung mit Zeitbereichen, die ein Event bei nderung liefert
''' </summary>
Public Class TimeAreaList
    Inherits List(Of TimeArea)

    ' aktuell selektierter Zeitbereich
    Private m_SelectedBereich As TimeArea = Nothing

    ''' <summary>
    ''' Event bei nderung der Zeitbereiche
    ''' </summary>
    Public Event Changed()


    ''' <summary>
    ''' gibt den aktuell selektierten Zeitbereich zurck oder legt diesen fest
    ''' </summary>
    Property SelectedBereich() As TimeArea
        Get
            Return m_SelectedBereich
        End Get
        Set(ByVal value As TimeArea)
            m_SelectedBereich = value
        End Set
    End Property


    ''' <summary>
    ''' fgt einen Zeitbereich hinzu
    ''' </summary>
    Shadows Sub Add(ByVal Bereich As TimeArea)
        MyBase.Add(Bereich)
        ' der hinzugefgte Zeitbereich wird zum selektierten
        m_SelectedBereich = Bereich
        ' sich berlappende Zeitbereiche zu einem verschmelzen
        Merge()
        ' nderungs-Event liefern
        RaiseEvent Changed()
    End Sub

    ''' <summary>
    ''' entfernt einen Zeitbereich
    ''' </summary>
    Shadows Function Remove(ByVal Bereich As TimeArea)

        If m_SelectedBereich Is Bereich Then
            ' wenn der zu entfernende Bereich der aktuell selektierte ist,
            ' falls vorhanden, neu selektierten Bereich festlegen
            Dim Idx As Integer = Me.IndexOf(Bereich)
            If Idx > 0 Then
                m_SelectedBereich = Me(Idx - 1)
            ElseIf Me.Count - 1 > Idx Then
                m_SelectedBereich = Me(Idx + 1)
            Else
                m_SelectedBereich = Nothing
            End If
        End If

        ' den bisher selektierten Bereich entfernen
        Dim Result As Boolean = MyBase.Remove(Bereich)

        ' nderungs-Event liefern
        RaiseEvent Changed()
        Return Result
    End Function

    ''' <summary>
    ''' verbindet sich berlappende Zeitbereiche zu einem gemeinsamen
    ''' </summary>
    Private Sub Merge()
        Me.Sort(New ZeitBereichsComparer)
        If Me.Count > 1 Then
            For I As Integer = Me.Count - 1 To 1 Step -1
                If Me(I).Start <= Me(I - 1).Ende Then
                    If Me(I).Ende > Me(I - 1).Ende Then
                        Dim Interval As New TimeSpan(Me(I).Ende.Ticks - Me(I - 1).Start.Ticks)
                        Me(I - 1).Length = Interval.TotalMinutes
                    End If
                    If m_SelectedBereich Is Me(I) Then
                        m_SelectedBereich = Me(I - 1)
                    End If
                    Me.RemoveAt(I)
                End If
            Next
        End If
    End Sub

    ''' <summary>
    ''' Comparer zum Sortieren einer Zeitbereichsliste
    ''' </summary>
    Private Class ZeitBereichsComparer
        Implements IComparer(Of TimeArea)

        Public Function Compare(ByVal x As TimeArea, ByVal y As TimeArea) As Integer Implements System.Collections.Generic.IComparer(Of TimeArea).Compare
            If x.Start = y.Start Then
                Return x.Ende.CompareTo(y.Ende)
            Else
                Return x.Start.CompareTo(y.Start)
            End If

        End Function
    End Class
End Class

